﻿#region LicenseHeader

// Copyright 2012 The Trustees of Indiana University.  All rights reserved.
// 
// The Indiana University licenses this file to you under the Apache License, 
// Version 2.0 (the "License"); you may not use this file except in compliance 
// with the License.  You may obtain a copy of the License at 
// 
// http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing, software 
// distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
//  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  
// See the License for the specific language governing permissions and 
// limitations under the License.
// 
// @author Thilina Gunarathne (tgunarat@indiana.edu)

#endregion

#region

using System;

#endregion

namespace AzureMRCore.DataTypes
{
    public class DoubleValue : IValue
    {
        public double Value { get; set; }

        #region IValue Members

        public string GetTextValue()
        {
            return Value.ToString();
        }

        public Boolean ParseValue(object value)
        {
            if (value is string)
            {
                double val;
                Boolean success = Double.TryParse((string) value, out val);
                Value = val;
                return success;
            }
            return false;
        }

        public byte[] GetBytes()
        {
            return BitConverter.GetBytes(Value);
        }

        public bool FromBytes(byte[] value)
        {
            Value = BitConverter.ToDouble(value, 0);
            return true;
        }

        public long Size()
        {
            return 8;
        }

        #endregion

        public static DoubleValue GetInstance(double value)
        {
            DoubleValue instance = new DoubleValue {Value = value};
            return instance;
        }

        public override string ToString()
        {
            return GetTextValue();
        }
    }
}