﻿#region LicenseHeader

// Copyright 2012 The Trustees of Indiana University.  All rights reserved.
// 
// The Indiana University licenses this file to you under the Apache License, 
// Version 2.0 (the "License"); you may not use this file except in compliance 
// with the License.  You may obtain a copy of the License at 
// 
// http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing, software 
// distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
//  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  
// See the License for the specific language governing permissions and 
// limitations under the License.
// 
// @author Thilina Gunarathne (tgunarat@indiana.edu)

#endregion

#region

using System;
using System.Collections.Generic;
using System.IO;
using AzureMRCore.DataTypes;
using AzureMRCore.Primitives;

#endregion

namespace AzureMRCore.InputFormat
{
    public class FileInputFormat : IInputFormat<StringKey, StringValue>
    {
        private FileInfo file;
        private Boolean isDone;
        private string key;

        #region IInputFormat<StringKey,StringValue> Members

        public void Init(string inputFile)
        {
            string[] parts = inputFile.Split('/');
            string localFileName = parts[parts.Length - 1];
            key = localFileName;

            file = new FileInfo(inputFile);
        }

        public StringKey GetCurrentKey()
        {
            StringKey stringKey = new StringKey();
            stringKey.key = key;
            return stringKey;
        }

        public StringValue GetCurrentValue()
        {
            StringValue stringValue = new StringValue();
            stringValue.Value = file.FullName;
            return stringValue;
        }

        public bool Next()
        {
            if (isDone)
            {
                return false;
            }
            else
            {
                isDone = true;
                return true;
            }
        }

        public bool HasNext()
        {
            return !(isDone);
        }

        public void Close()
        {
        }

        public List<KeyValuePair<StringKey, StringValue>> GetAllPairs()
        {
            throw new NotImplementedException();
        }

        #endregion

        public void Init(IDictionary<int, BCastDataProduct> inputRecords)
        {
            throw new NotImplementedException();
        }
    }
}