﻿#region LicenseHeader

// Copyright 2012 The Trustees of Indiana University.  All rights reserved.
// 
// The Indiana University licenses this file to you under the Apache License, 
// Version 2.0 (the "License"); you may not use this file except in compliance 
// with the License.  You may obtain a copy of the License at 
// 
// http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing, software 
// distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
//  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  
// See the License for the specific language governing permissions and 
// limitations under the License.
// 
// @author Thilina Gunarathne (tgunarat@indiana.edu)

#endregion

#region

using System;
using System.Collections.Generic;
using System.IO;
using AzureMRCore.DataTypes;
using AzureMRCore.Primitives;

#endregion

namespace AzureMRCore.InputFormat
{
    public class LineInputFormat : IInputFormat<IntKey, StringValue>
    {
        private int counter;
        private string currentLine;
        private StreamReader reader;

        #region IInputFormat<IntKey,StringValue> Members

        public void Init(string inputFile)
        {
            reader = new StreamReader(inputFile);
        }

        public IntKey GetCurrentKey()
        {
            IntKey intKey = new IntKey();
            intKey.Key = counter;
            return intKey;
        }

        public StringValue GetCurrentValue()
        {
            StringValue value = new StringValue();
            value.Value = currentLine;
            return value;
        }

        public bool Next()
        {
            string line = reader.ReadLine();
            if (line != null)
            {
                currentLine = line;
                counter++;
                return true;
            }
            return false;
        }

        public bool HasNext()
        {
            return !(reader.EndOfStream);
        }

        public void Close()
        {
            reader.Close();
        }

        public List<KeyValuePair<IntKey, StringValue>> GetAllPairs()
        {
            throw new NotImplementedException();
        }

        #endregion

        public void Init(IDictionary<int, BCastDataProduct> inputRecords)
        {
            throw new NotImplementedException();
        }
    }
}