﻿#region LicenseHeader

// Copyright 2012 The Trustees of Indiana University.  All rights reserved.
// 
// The Indiana University licenses this file to you under the Apache License, 
// Version 2.0 (the "License"); you may not use this file except in compliance 
// with the License.  You may obtain a copy of the License at 
// 
// http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing, software 
// distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
//  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  
// See the License for the specific language governing permissions and 
// limitations under the License.
// 
// @author Thilina Gunarathne (tgunarat@indiana.edu)

#endregion

#region

using System.Collections.Generic;

#endregion

namespace AzureMRCore.IterativeDataCache
{
    public class MapTaskHistory
    {
        private readonly List<string> _childTasks;

        public MapTaskHistory()
        {
            _childTasks = new List<string>();
        }

        public string BlobURL { get; set; }
        public string JobID { get; set; }
        public string MapID { get; set; }
        public string AppName { get; set; }
        public string Iteration { get; set; }
        public int NumTotalMapTasks { get; set; }

        public bool IsChild(string jobID, string appName, string iteration)
        {
            var id = jobID + "_" + appName + "_" + iteration;
            return _childTasks.Contains(id);
        }


        public bool AddChild(string jobID, string appName, string iteration)
        {
            lock (_childTasks)
            {
                var id = jobID + "_" + appName + "_" + iteration;
                if (_childTasks.Contains(id))
                {
                    return false;
                }
                _childTasks.Add(id);
                return true;
            }
        }

        public void RemoveChild(string jobID, string appName, string iteration)
        {
            lock (_childTasks)
            {
                var id = jobID + "_" + appName + "_" + iteration;
                _childTasks.Remove(id);
            }
        }
    }
}