﻿#region LicenseHeader

// Copyright 2012 The Trustees of Indiana University.  All rights reserved.
// 
// The Indiana University licenses this file to you under the Apache License, 
// Version 2.0 (the "License"); you may not use this file except in compliance 
// with the License.  You may obtain a copy of the License at 
// 
// http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing, software 
// distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
//  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  
// See the License for the specific language governing permissions and 
// limitations under the License.
// 
// @author Thilina Gunarathne (tgunarat@indiana.edu)

#endregion

using System;
using System.Collections.Concurrent;
using System.Collections.Generic;
using System.IO;
using AzureMRCore.DataTypes;
using Microsoft.WindowsAzure.StorageClient;

namespace AzureMRCore.OutputFormat
{
    public class FileOutputFormat<K> : IOutputFormat<K, FileValue>
        where K : Key, new()
    {
        #region IOutputFormat<K,FileValue> Members

        public Dictionary<string, K> UploadValues(List<KeyValuePair<K, FileValue>> pairs, object firstID,
                                                  string iteration,
                                                  CloudBlobContainer container)
        {
            Dictionary<string, K> references = new Dictionary<string, K>();
            foreach (KeyValuePair<K, FileValue> pair in pairs)
            {
                CloudBlockBlob resultBlob =
                    container.GetBlockBlobReference("Out_" + firstID + "_" + iteration + "_" + pair.Key.GetTextValue());
                resultBlob.UploadFile(pair.Value.GetTextValue());
                references.Add(resultBlob.Uri.AbsoluteUri, pair.Key);
            }
            return references;
        }

        public long GetSize(List<KeyValuePair<K, FileValue>> pairs)
        {
            //disabled
            return long.MaxValue;
        }

        public byte[] ToBytes(List<KeyValuePair<K, FileValue>> pairs)
        {
            throw new NotImplementedException();
        }

        public void FromBytes(byte[] values, ConcurrentDictionary<K, List<FileValue>> decodedData)
        {
            throw new NotImplementedException();
        }

        public void DecodeData(string fileName, string key, ConcurrentDictionary<K, List<FileValue>> decodedData)
        {
            K rowKey = new K();
            rowKey.Parse(key);
            FileValue rowValue = FileValue.GetInstance(new FileInfo(fileName));

            List<FileValue> values = decodedData.GetOrAdd(rowKey, new List<FileValue>());
            lock (values)
            {
                values.Add(rowValue);
            }
        }

        #endregion
    }
}