﻿#region LicenseHeader

// Copyright 2012 The Trustees of Indiana University.  All rights reserved.
// 
// The Indiana University licenses this file to you under the Apache License, 
// Version 2.0 (the "License"); you may not use this file except in compliance 
// with the License.  You may obtain a copy of the License at 
// 
// http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing, software 
// distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
//  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  
// See the License for the specific language governing permissions and 
// limitations under the License.
// 
// @author Thilina Gunarathne (tgunarat@indiana.edu)

#endregion

#region

using System;
using System.Collections.Generic;
using AzureMRCore;
using AzureMRCore.DataTypes;
using AzureMRCore.MapRed;

#endregion

namespace KMeansMRSample
{
    internal class KmeansCombiner : Reducer<IntKey, DoubleVectorValue, IntKey, DoubleVectorValue>
    {
        public override int Reduce(IntKey key, List<DoubleVectorValue> values, IOutputCollector<IntKey, DoubleVectorValue> outputCollector, string programArgs)
        {
            int vectorLength;
            bool vecLengthSuccess = Int32.TryParse(programArgs, out vectorLength);
            if (!vecLengthSuccess)
            {
                vectorLength = values[0].Value.Length;
            }


            var newCentroid = new double[vectorLength + 1];
            double count = 0;

            foreach (DoubleVectorValue value in values)
            {
                for (int i = 0; i < vectorLength; i++)
                {
                    newCentroid[i] += value.Value[i];
                }
                count++;
            }
            newCentroid[vectorLength] = count;
            DoubleVectorValue outValue = DoubleVectorValue.GetInstance(newCentroid);
            outputCollector.Collect(key, outValue);
            return 0;
        }
    }
}