﻿#region LicenseHeader

// Copyright 2012 The Trustees of Indiana University.  All rights reserved.
// 
// The Indiana University licenses this file to you under the Apache License, 
// Version 2.0 (the "License"); you may not use this file except in compliance 
// with the License.  You may obtain a copy of the License at 
// 
// http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing, software 
// distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
//  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  
// See the License for the specific language governing permissions and 
// limitations under the License.
// 
// @author Thilina Gunarathne (tgunarat@indiana.edu)

#endregion

using AzureMRCore;
using AzureMRCore.DataTypes;
using AzureMRCore.Drivers;
using AzureMRCore.InputFormat;
using AzureMRCore.MapRed;
using AzureMRCore.OutputFormat;
using AzureMRCore.Partitioners;
using Microsoft.WindowsAzure;

namespace KMeansMRSample
{
    public class KmeansMR :
        IterativeMRDriver
            <IntKey, MatrixValue<double>, IntKey, DoubleVectorValue, IntKey, DoubleVectorValue, IntKey,
            MatrixValue<double>,
            IntKey, MatrixValue<double>>
    {
        public KmeansMR()
        {
            DirectTCPTranfer = 2;
        }

        public override bool ConfigMapRed(MRConf mrConf, CloudStorageAccount storageAccount, out object configOut)
        {
            configOut = null;
            return true;
        }

        public override string Name
        {
            get { return "KMeansMR"; }
        }

        public override Mapper<IntKey, MatrixValue<double>, IntKey, DoubleVectorValue, IntKey, MatrixValue<double>>
            MapperType
        {
            get { return new KmeansMatrixMapper(); }
        }

        public override Reducer<IntKey, DoubleVectorValue, IntKey, DoubleVectorValue> ReducerType
        {
            get { return new KmeansReducer(); }
        }

        public override Merger<IntKey, DoubleVectorValue, IntKey, MatrixValue<double>, IntKey, MatrixValue<double>>
            MergeType
        {
            get { return new KMeansMerge(); }
        }

        public override IInputFormat<IntKey, MatrixValue<double>> InputFormatType
        {
            get { return new CachedDoubleMatrixFormat(); }
        }

        public override IOutputFormat<IntKey, DoubleVectorValue> MapOutputFormatType
        {
            get { return new DoubleVectorOutputFormat(); }
        }

        public override IOutputFormat<IntKey, DoubleVectorValue> ReduceOutputFormatType
        {
            get { return new DoubleVectorOutputFormat(); }
        }

        public override IOutputFormat<IntKey, MatrixValue<double>> MergeOutputFormatType
        {
            get { return new DoubleMatrixOutputFormat(); }
        }

        public override IPartitioner PartitionerType
        {
            get { return new HashPartitioner(); }
        }
    }
}