﻿#region LicenseHeader

// Copyright 2012 The Trustees of Indiana University.  All rights reserved.
// 
// The Indiana University licenses this file to you under the Apache License, 
// Version 2.0 (the "License"); you may not use this file except in compliance 
// with the License.  You may obtain a copy of the License at 
// 
// http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing, software 
// distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
//  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  
// See the License for the specific language governing permissions and 
// limitations under the License.
// 
// @author Thilina Gunarathne (tgunarat@indiana.edu)

#endregion

#region

using System.Collections.Generic;
using AzureMRCore;
using AzureMRCore.DataTypes;
using AzureMRCore.MapRed;

#endregion

namespace KMeansMRSample
{
    internal class KmeansMatrixMapper :
        Mapper<IntKey, MatrixValue<double>, IntKey, DoubleVectorValue, IntKey, MatrixValue<double>>
    {
        protected override int Map(IntKey key, MatrixValue<double> matrixValue, List<KeyValuePair<IntKey, MatrixValue<double>>> dynamicData, IOutputCollector<IntKey, DoubleVectorValue> outputCollector, string programArgs)
        {
            double[][] centroids = dynamicData[0].Value.Matrix;
            int vectorLength = dynamicData[0].Value.Width;

            double[][] newCentroids = GetNewCentroids(centroids, vectorLength, matrixValue);

            //double[][] newCentroids2 = GetNewCentroids(centroids, vectorLength, matrixValue);

            for (int i = 0; i < newCentroids.Length; i++)
            {
                outputCollector.Collect(IntKey.GetInstance(i), DoubleVectorValue.GetInstance(newCentroids[i]));
            }

            return 0;
        }

        private double[][] GetNewCentroids(double[][] centroids, int vectorLength, MatrixValue<double> matrixValue)
        {
            double[][] newCentroids = new double[centroids.Length][];

            for (int i = 0; i < newCentroids.Length; i++)
            {
                newCentroids[i] = new double[vectorLength + 1];
            }

            double[][] valueMatrix = matrixValue.Matrix;
            int numCentroids = centroids.Length;

            foreach (double[] t in valueMatrix)
            {
                double min = 0;
                int minCentroid = 0;
                for (int j = 0; j < numCentroids; j++)
                {
                    double euDistance = 0;
                    for (int i = 0; i < vectorLength; i++)
                    {
                        double distance = (centroids[j][i] - t[i]);
                        euDistance += distance*distance;
                    }

                    if (j == 0)
                    {
                        min = euDistance;
                    }
                    else if (euDistance < min)
                    {
                        min = euDistance;
                        minCentroid = j;
                    }
                }
                for (int i = 0; i < vectorLength; i++)
                {
                    newCentroids[minCentroid][i] += t[i];
                }
                newCentroids[minCentroid][vectorLength] += 1;
            }
            return newCentroids;
        }
    }
}