﻿#region LicenseHeader

// Copyright 2012 The Trustees of Indiana University.  All rights reserved.
// 
// The Indiana University licenses this file to you under the Apache License, 
// Version 2.0 (the "License"); you may not use this file except in compliance 
// with the License.  You may obtain a copy of the License at 
// 
// http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing, software 
// distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
//  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  
// See the License for the specific language governing permissions and 
// limitations under the License.
// 
// @author Thilina Gunarathne (tgunarat@indiana.edu)

#endregion

#region

using System;
using System.Collections.Generic;
using System.Linq;
using AzureMRCore;
using AzureMRCore.DataTypes;
using AzureMRCore.MapRed;
using Microsoft.WindowsAzure.ServiceRuntime;

#endregion

namespace KMeansMRSample
{
    internal class KmeansReducer : Reducer<IntKey, DoubleVectorValue, IntKey, DoubleVectorValue>
    {
        public override int Reduce(IntKey key, List<DoubleVectorValue> values, IOutputCollector<IntKey, DoubleVectorValue> outputCollector, string programArgs)
        {
            double[] newCentroid = generateNewCentroid(programArgs, values);

            DoubleVectorValue outValue = DoubleVectorValue.GetInstance(newCentroid);
            outputCollector.Collect(key, outValue);
            return 0;
        }

        private double[] generateNewCentroid(string programArgs, List<DoubleVectorValue> values)
        {
            int vectorLength;
            bool isAccumulated = false;
            bool vecLengthSuccess = Int32.TryParse(programArgs, out vectorLength);
            if (!vecLengthSuccess)
                vectorLength = values[0].Value.Length;
            else
                isAccumulated = vectorLength < values[0].Value.Length;

            var newCentroid = new double[vectorLength];
            double count = 0;

            foreach (var value in values)
            {
                for (int i = 0; i < vectorLength; i++)
                {
                    if (value != null)
                    {
                        newCentroid[i] += value.Value[i];
                    }
                    else
                    {
                        throw new Exception("Value is null." + i + "  " + count + "  " + vectorLength + " " +
                                            isAccumulated + " " + RoleEnvironment.CurrentRoleInstance.Id);
                    }
                }
                if (isAccumulated)
                    count += value.Value.Last();
                else
                    count++;
            }
            for (int i = 0; i < vectorLength; i++)
            {
                newCentroid[i] = newCentroid[i]/count;
            }
            return newCentroid;
        }
    }
}