﻿#region LicenseHeader

// Copyright 2012 The Trustees of Indiana University.  All rights reserved.
// 
// The Indiana University licenses this file to you under the Apache License, 
// Version 2.0 (the "License"); you may not use this file except in compliance 
// with the License.  You may obtain a copy of the License at 
// 
// http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing, software 
// distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
//  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  
// See the License for the specific language governing permissions and 
// limitations under the License.
// 
// @author Thilina Gunarathne (tgunarat@indiana.edu)

#endregion

using System;
using System.Diagnostics;
using AzureMRCore.Client;
using Credentials;
using Microsoft.WindowsAzure;
using Microsoft.WindowsAzure.StorageClient;

namespace PageRankMRClient
{
    internal class PageRankClient
    {
        // args : num_seq block_size maxBlocks jobname
        private static void Main(string[] args)
        {
            CloudStorageAccount storageAccount = ClientCredentials.GetClientStorageAccount();

            if (args.Length < 6)
            {
                Console.Out.WriteLine(
                    "Usage:  PageRankClient <jobID> <inputContainerName> <outContainerName> <NumberOfReduceTasks> <Cluster Centers> <vector length>");
            }
            string jobID = args[0];
            string inputContainer = args[1];
            string outputContainer = args[2];
            int numReduceTasks = Int32.Parse(args[3]);
            string broadcastURI = args[4];
            string vectorLength = args[5];

            CloudTableClient tableClient = storageAccount.CreateCloudTableClient();

            Stopwatch stopwatch = Stopwatch.StartNew();
            TwisterAzureClient twisterAzureClient = new TwisterAzureClient(storageAccount, jobID, "mapsched4",
                                                                           "reducesched4");
            twisterAzureClient.ProcessMapRed("PageRankMR", inputContainer, 0, vectorLength, numReduceTasks,
                                             outputContainer, broadcastURI, true);
            twisterAzureClient.WaitForCompletion(500);
            Console.Out.WriteLine("Time (ms) " + stopwatch.ElapsedMilliseconds);
            Console.In.ReadLine();
        }
    }
}