﻿#region LicenseHeader

// Copyright 2012 The Trustees of Indiana University.  All rights reserved.
// 
// The Indiana University licenses this file to you under the Apache License, 
// Version 2.0 (the "License"); you may not use this file except in compliance 
// with the License.  You may obtain a copy of the License at 
// 
// http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing, software 
// distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
//  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  
// See the License for the specific language governing permissions and 
// limitations under the License.
// 
// @author Thilina Gunarathne (tgunarat@indiana.edu)

#endregion

#region

using AzureMRCore.DataTypes;
using AzureMRCore.Drivers;
using AzureMRCore.InputFormat;
using AzureMRCore.MapRed;
using AzureMRCore.OutputFormat;
using AzureMRCore.Partitioners;

#endregion

namespace PageRankMR
{
    internal class PageRankMR :
        IterativeMRDriver
            <IntKey, AdjacencyListValue<int>, IntKey, DoubleValue, IntKey, DoubleValue, IntKey, DoubleVectorValue,
            IntKey, DoubleVectorValue>
    {
        public const double DANGLING_PROBABILITY = 0.85;

        public override string Name
        {
            get { return "PageRankMR"; }
        }

        public override IOutputFormat<IntKey, DoubleValue> MapOutputFormatType
        {
            get { return new SequenceOutputFormat<IntKey, DoubleValue>(); }
        }

        public override IOutputFormat<IntKey, DoubleValue> ReduceOutputFormatType
        {
            get { return new SequenceOutputFormat<IntKey, DoubleValue>(); }
        }

        public override Mapper<IntKey, AdjacencyListValue<int>, IntKey, DoubleValue, IntKey, DoubleVectorValue>
            MapperType
        {
            get { return new PageRankMapper(); }
        }

        // TODO
        public override IInputFormat<IntKey, AdjacencyListValue<int>> InputFormatType
        {
            get { return null; }
        }

        public override Reducer<IntKey, DoubleValue, IntKey, DoubleValue> ReducerType
        {
            get { return new PageRankReducer(); }
        }

        public override Merger<IntKey, DoubleValue, IntKey, DoubleVectorValue, IntKey, DoubleVectorValue> MergeType
        {
            get { return new PageRankMerge(); }
        }

        public override Reducer<IntKey, DoubleValue, IntKey, DoubleValue> CombinerType
        {
            get { return new PageRankReducer(); }
        }

        public override IPartitioner PartitionerType
        {
            get { return new HashPartitioner(); }
        }
    }
}