#region LicenseHeader

// Copyright 2012 The Trustees of Indiana University.  All rights reserved.
// 
// The Indiana University licenses this file to you under the Apache License, 
// Version 2.0 (the "License"); you may not use this file except in compliance 
// with the License.  You may obtain a copy of the License at 
// 
// http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing, software 
// distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
//  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  
// See the License for the specific language governing permissions and 
// limitations under the License.
// 
// @author Thilina Gunarathne (tgunarat@indiana.edu)

#endregion

#region

using System.Collections;

#endregion

namespace NAligner.Formats
{
    /// <summary>
    ///   Singleton formatter factory.
    /// </summary>
    public class FormatterFactory
    {
        private static FormatterFactory singletonInstance;

        /// <summary>
        ///   Table containing the <see cref="AlignmentFormatter" /> s.
        /// </summary>
        private readonly Hashtable formats = new Hashtable();

        /// <summary>
        ///   Hidden constructor
        /// </summary>
        private FormatterFactory()
        {
        }

        /// <summary>
        ///   Returns the singleton instance for <see cref="FormatterFactory" /> .
        /// </summary>
        public static FormatterFactory Default
        {
            get
            {
                if (singletonInstance == null)
                {
                    singletonInstance = new FormatterFactory();
                }
                return singletonInstance;
            }
        }

        /// <summary>
        ///   Returns a list of registered formatters
        /// </summary>
        public virtual ICollection Formatters
        {
            get { return formats.Keys; }
        }

        /// <summary>
        ///   Registers a new formatter.
        /// </summary>
        public virtual void RegisterFormatter(AlignmentFormatter alignmentFormatter)
        {
            formats[alignmentFormatter.Id] = alignmentFormatter;
        }

        /// <summary>
        ///   Returns a <see cref="AlignmentFormatter" /> .
        /// </summary>
        /// <param name="id"> formatter id </param>
        public virtual AlignmentFormatter GetFormatter(string id)
        {
            return (AlignmentFormatter) formats[id];
        }
    }
}