﻿#region LicenseHeader

// Copyright 2012 The Trustees of Indiana University.  All rights reserved.
// 
// The Indiana University licenses this file to you under the Apache License, 
// Version 2.0 (the "License"); you may not use this file except in compliance 
// with the License.  You may obtain a copy of the License at 
// 
// http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing, software 
// distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
//  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  
// See the License for the specific language governing permissions and 
// limitations under the License.
// 
// @author Thilina Gunarathne (tgunarat@indiana.edu)

#endregion

using AzureMRCore.DataTypes;
using AzureMRCore.Drivers;
using AzureMRCore.InputFormat;
using AzureMRCore.MapRed;
using AzureMRCore.OutputFormat;
using AzureMRCore.Partitioners;

namespace SWGMBFSample
{
    public class SWGDriver : MapReduceDriver<IntKey, StringValue, IntKey, FileValue, IntKey, FileValue>
    {
        public override string Name
        {
            get { return "SWGMBFMR"; }
        }

        public override Mapper<IntKey, StringValue, IntKey, FileValue, NullKey, NullValue> MapperType
        {
            get { return new SWGMapper(); }
        }

        public override Reducer<IntKey, FileValue, IntKey, FileValue> ReducerType
        {
            get { return new SWGReducer(); }
        }

        public override IInputFormat<IntKey, StringValue> InputFormatType
        {
            get { return new LineInputFormat(); }
        }

        public override IPartitioner PartitionerType
        {
            get { return new IdentityPartitioner(); }
        }

        public override IOutputFormat<IntKey, FileValue> MapOutputFormatType
        {
            get { return new FileOutputFormat<IntKey>(); }
        }

        public override IOutputFormat<IntKey, FileValue> ReduceOutputFormatType
        {
            get { return new FileOutputFormat<IntKey>(); }
        }
    }
}