﻿#region LicenseHeader

// Copyright 2012 The Trustees of Indiana University.  All rights reserved.
// 
// The Indiana University licenses this file to you under the Apache License, 
// Version 2.0 (the "License"); you may not use this file except in compliance 
// with the License.  You may obtain a copy of the License at 
// 
// http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing, software 
// distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
//  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  
// See the License for the specific language governing permissions and 
// limitations under the License.
// 
// @author Thilina Gunarathne (tgunarat@indiana.edu)

#endregion

#region

using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.IO;
using AzureMRCore;
using AzureMRCore.DataModel;
using AzureMRCore.DataTypes;
using AzureMRCore.MapRed;
using Microsoft.WindowsAzure;
using Microsoft.WindowsAzure.ServiceRuntime;
using Microsoft.WindowsAzure.StorageClient;

#endregion

namespace BlastWorkerSample
{
    public class BlastMapper : Mapper<StringKey, StringValue, StringKey, FileValue, NullKey, NullValue>
    {
        private string _dbPath;
        private CloudStorageAccount _storageAccount;

        /*
         * Run once when starting the MapWorker.
         */

        protected override Boolean Config(CloudStorageAccount storageAccount, object configIn)
        {
            _storageAccount = storageAccount;
            _dbPath = (string) configIn;
            return true;
        }

        private void Log(string message, CloudStorageAccount storageAccount)
        {
            CloudTableClient tableClient = storageAccount.CreateCloudTableClient();
            ErrorDataContext errorContext = new ErrorDataContext(storageAccount.TableEndpoint.ToString(),
                                                                 storageAccount.Credentials, "AzureMRErrors");
            tableClient.CreateTableIfNotExist(errorContext.Name);
            ErrorDataModel error =
                new ErrorDataModel(RoleEnvironment.CurrentRoleInstance.Id + "_  " + message);
            errorContext.AddObject(errorContext.Name, error);
            errorContext.SaveChangesWithRetries();
        }

        protected override int Map(StringKey key, StringValue value, List<KeyValuePair<NullKey, NullValue>> dynamicData, IOutputCollector<StringKey, FileValue> outputCollector, string programArgs)
        {
            if (programArgs == null)
            {
                programArgs = "1";
            }
            string outFile = value + "out";
            string cmdArgs = string.Format(" -query " + value + " -db " +
                                           _dbPath + " -outfmt 6 -seg no  -out " + outFile + " -num_threads " +
                                           programArgs);

            Stopwatch sw = Stopwatch.StartNew();
            Process process = new Process();
            ProcessStartInfo startInfo = process.StartInfo;
            startInfo.Arguments = cmdArgs;
            startInfo.FileName = Environment.CurrentDirectory + "\\blast\\blastx.exe";
            startInfo.WorkingDirectory = Path.GetDirectoryName(startInfo.FileName);

            //startInfo.RedirectStandardOutput = true;
            startInfo.UseShellExecute = false;
            startInfo.CreateNoWindow = true;
            //TODO : handle the error output
            startInfo.RedirectStandardError = false;
            startInfo.RedirectStandardOutput = true;

            process.Start();
            string output = process.StandardOutput.ReadToEnd();

            process.WaitForExit();
            Log(output, _storageAccount);
            int exitCode = process.ExitCode;
            sw.Stop();
            Log("exitcode:" + exitCode + " TotalProc "
                //+process.TotalProcessorTime+"  UserPro "+process.UserProcessorTime+" "
                + sw.ElapsedMilliseconds + "  " + programArgs, _storageAccount);

            outputCollector.Collect(key, FileValue.GetInstance(new FileInfo(outFile)));

            return 0;
        }
    }
}